##  TAR_UR.R
##  
##  This R procedure estimates a TAR(p+1).
##  It tests for a threshold and for unit roots.
##  
##  written by:
##  
##  Bruce E. Hansen
##  Department of Economics
##  Social Science Building
##  University of Wisconsin
##  Madison, WI 53706-1393
##  behansen@wisc.edu
##  http://www.ssc.wisc.edu/~bhansen/
##  
##  This procedure implements the estimation and testing methods 
##  described in "Threshold Autoregression with a Near Unit Root"
##  by Mehmet Caner and Bruce E. Hansen.
##  
##  ***************************************************************** 
##
##  Format:
##  
##  output <- tar_ur(y,p,mmin,mmax,m)
##  
##  Inputs:
##  
##  y         vector of data
##  p         # of AR lags, must be greater or equal to one
##  mmin      Minimal Delay Order for testing
##  mmax      Maximal Delay Order for testing
##  m		  Delay Order (fixed) for estimation
##  		  Set to 0 for m to be estimated by minimizing SSE
##  
##  There are no outputs.  Results are printed to the screen.
##  
##  Global Controls:
##  
##  thresh    Type of threshold variable z
##  		  1 for long difference, z(t-1)=y(t-1)-y(t-(m-1))
##  		  2 for lagged difference, z(t-1)=y(t-m)-y(t-m-1)
##  		  3 for lagged level, z(t-1)=y(t-m)
##  trim      Trimming Range (normalized threshold, lambda)
##  		  1 for [.15,.85]
##  	 	  2 for [.10,.90]
##  		  3 for [.05,.95]
##  trend	  1 to include linear time trend
##  		  0 to exclude linear time trend
##  graph     set to 1 to display graphs, else 0
##  xdate     nx1 vector, equal in length to dependent variable, 
##            for displaying predicted values in graph
##  	        For default of undated x-axis, set xdate=0;
##  boot      # of bootstrap replications (1 or greater)
##  adj	  Adjustment to Gaussian likelihood for Jacobian transformations
##  test	  AR lag indices to jointly test for equality (optional)
##  		  Set to 0 for no test
##  		
###########################################################################

## Default Global Settings ##
thresh <- 1
trim <- 1
graph <- 1
trend <- 0
boot <- 100
adj <- 0
xdate <- 0
test <- 0

#**********************************************#
#  Main Procedure                              #
#**********************************************#

tar_ur <- function(dat,p,mmin,mmax,m){  

t <- nrow(dat)
n <- t - 1 - p
mn <- mmax-mmin+1
ms <- as.matrix(seq(mmin,mn,1))

## Linear Model ##
dy <- as.matrix(dat[2:t]-dat[1:(t-1)])
y <- as.matrix(dy[(1+p):(t-1)])
x <- matrix(1,n,1)
if (trend==1) x <- cbind(x,as.matrix(seq(1,n,1)))
x <- cbind(x,dat[(1+p):(t-1)],dy[p:(t-2)])
for (k in 2:p) x <- cbind(x,dy[(p+1-k):(t-1-k)])
kx <- ncol(x)
xx <- solve(t(x)%*%x)
bols <- xx%*%(t(x)%*%y)
eols <- y - x%*%bols
sols <- t(eols)%*%eols
sigols <- sols/(n-kx)
seols <- sqrt(diag(xx)%*%sigols)
rho <- bols[2+trend]
tadf <- rho/seols[2+trend]
ar0 <- as.matrix(bols[(3+trend):(2+trend+p)])

#*****************************#
#  Computation Procedure      #
#*****************************#

tur_est <- function(dat,p,mmin,mmax,m){
  t <- nrow(dat)
  n <- t - 1 - p
  mn <- mmax-mmin+1
  ms <- as.matrix(seq(mmin,mn,1))
  dy <- as.matrix(dat[2:t]-dat[1:(t-1)])
  y <- as.matrix(dy[(1+p):(t-1)])
  x <- matrix(1,n,1)
  if (trend==1) x <- cbind(x,as.matrix(seq(1,n,1)))
  x <- cbind(x,dat[(1+p):(t-1)],dy[p:(t-2)])
  for (ki in 2:p) x <- cbind(x,dy[(p+1-ki):(t-1-ki)])
  if (thresh==1){
      qs <- dat[(1+p):(t-1)]-dat[(1+p-mmin):(t-1-mmin)]
      qs <- as.matrix(qs)
      for (mi in (mmin+1):mmax){
          qs <- cbind(qs,(dat[(1+p):(t-1)]-dat[(1+p-mi):(t-1-mi)]))
      }
  }
  if (thresh==2){
      qs <- dat[(1+p-mmin+1):(t-1-mmin+1)]-dat[(1+p-mmin):(t-1-mmin)]
      qs <- as.matrix(qs)
      for (mi in (mmin+1):mmax){
          qs <- cbind(qs,(dat[(1+p-mi+1):(t-1-mi+1)]-dat[(1+p-mi):(t-1-mi)]))
      }
  }
  if (thresh==3){
      qs <- dat[(1+p-mmin+1):(t-mmin)]
      qs <- as.matrix(qs)
      for (mi in (mmin+1):mmax){
          qs <- cbind(qs,(dat[(1+p-mi+1):(t-mi)]))
      }
  }
  kx <- ncol(x)
  xx <- solve(t(x)%*%x)  
  bols <- xx%*%(t(x)%*%y)
  eols <- y - x%*%bols
  sols <- t(eols)%*%eols
  smins <- matrix(0,mn,1) 
  lams <- matrix(0,mn,1)
  ws   <- matrix(0,mn,1)
  r1   <- matrix(0,mn,1)
  r2   <- matrix(0,mn,1)
  t1   <- matrix(0,mn,1)
  t2   <- matrix(0,mn,1)
  xx <- x%*%xx
  rur <- rbind(2,(kx+2))+trend
  pi1 <-.2-.05*trim
  pi2 <-.8+.05*trim
  pi_n <- round((pi2-pi1)*100)+1
  for (mi in 1:mn){
    q <- qs[,mi]
    qq <- unique(q)
    qq <- as.matrix(sort(qq))
    rq <- nrow(qq)
    if (rq>100){
      qq <- as.matrix(qq[round(seq(pi1,pi1+(pi_n-1)*.01,.01)*rq)])
    }else{
      qq <- as.matrix(qq[floor(n*pi1):ceiling(n*pi2)])
    }
    qn <- nrow(qq)
    s <- matrix(0,qn,1)
    for (qi in 1:qn){ 
      d1 <- x*(as.matrix(q<qq[qi])%*%matrix(1,1,ncol(x)))
      d1 <- d1 - x%*%(t(xx)%*%d1)
      e  <- eols - d1%*%qr.solve(d1,eols) 
      s[qi] <- t(e)%*%e
    }
    qi <- which.min(s)
    lam <- qq[qi]
    smin <- s[qi]
    ws[mi] <- (n-2*kx)*(sols-smin)/smin
    smins[mi] <- smin
    lams[mi] <- lam
    d1 <- as.matrix((q<lam))
    d2 <- 1-d1
    xz <- cbind((x*(d1%*%matrix(1,1,ncol(x)))),(x*(d2%*%matrix(1,1,ncol(x)))))
    mmi <- solve(t(xz)%*%xz)
    bthresh  <- mmi%*%(t(xz)%*%y)
    e  <- y - xz%*%bthresh
    v <- mmi*as.vector((t(e)%*%e)/(n-2*kx))
    vd <- diag(v)
    ts <- as.matrix(bthresh[rur]/sqrt(vd[rur]))
    r1[mi] <- t(ts^2)%*%(ts<0)
    r2[mi] <- t(ts)%*%ts
    t1[mi] <- -ts[1]
    t2[mi] <- -ts[2]
  }
  mi <- which.min(smins)
  mhat <- ms[mi]
  w    <- ws[mi]
  if (m != 0) mi <- m-mmin+1 
  lam  <- lams[mi]
  q <- qs[,mi]
  d1 <- as.matrix((q<lam))
  d2 <- 1-d1
  xz <- cbind((x*(d1%*%matrix(1,1,ncol(x)))),(x*(d2%*%matrix(1,1,ncol(x)))))
  mmi <- solve(t(xz)%*%xz)
  bthresh  <- mmi%*%(t(xz)%*%y)
  e  <- y - xz%*%bthresh
  ee <- t(e)%*%e
  v <- mmi*as.vector((ee)/(n-2*kx))
  vd <- diag(v)
  v1 <- vd[1:kx]
  v2 <- vd[(kx+1):(2*kx)]
  b1 <- bthresh[1:kx]
  b2 <- bthresh[(kx+1):(2*kx)]
  ts <- as.matrix(((b1-b2)^2)/(v1+v2))
  bss <- cbind(b1,sqrt(v1),matrix(0,kx,1),b2,sqrt(v2))
  wi <- test+2+trend
  b1 <- as.matrix(bthresh[wi])
  b2 <- as.matrix(bthresh[kx+wi])
  v <- v[wi,wi]+v[(kx+wi),(kx+wi)]
  wtest <- t(b1-b2)%*%(qr.solve(v,(b1-b2)))
list(mhat=mhat,lam=lam,ee=ee,bss=bss,d1=d1,d2=d2,ws=ws,w=w,r1=r1,r2=r2,t1=t1,t2=t2,ts=ts,wtest=wtest)
}

## Threshold Model Estimation ##

out <- tur_est(dat,p,mmin,mmax,m)
mhat <- out$mhat
lam <- out$lam
ee <- out$ee
bss <- out$bss
d1 <- out$d1
d2 <- out$d2
ws <- out$ws
w <- out$w
r1 <- out$r1
r2 <- out$r2
t1 <- out$t1
t2 <- out$t2
ts <- out$ts
wtest <- out$wtest

## Graphs of Regime Classification ##

if (graph==1){
  if (xdate[1] == 0) xdate <- as.matrix(seq(1,n,1))
  if (nrow(xdate) != n){ 
      xdate <- as.matrix(seq(1,n,1))
  }else{ xdate <- xdate
  } 
  yp <- as.matrix(dat[(p+2):t])
  x11()
  xxlim <- range(xdate)
  yylim <- range(yp)
  plot(xdate[d1],yp[d1],pch=2,col=3,xlim=xxlim,ylim=yylim,type="p",ann=0)
  points(xdate[d2>0],yp[d2>0],pch=3,col=1)
  legend("bottomright",c("Regime 1","Regime 2"),pch=c(3,1),col=c(3,1))
}

## Bootstrap, Unconstrained Estimates ##

  y0 <- dat-mean(dat)
  y0 <- as.matrix(y0[1:(1+p)])
  aa <- matrix(0,p+1,1)
  aa[1] <- 1 + rho
  aa[1:p] <- aa[1:p] + ar0
  aa[2:(p+1)] <- aa[2:(p+1)] - ar0
  bootw <- matrix(0,boot,1)
  bootwt <- matrix(0,boot,1)
  bootws <- matrix(0,boot,mn)
  bootts <- matrix(0,boot,kx)
  for (ib in 1:boot){
    eols_c <- as.matrix(eols[ceiling(runif(t)*n)])
    datb <- y0
    for (i in (nrow(y0)+1):(nrow(eols_c))){
        datbb <- eols_c[i]
        for (j in 1:nrow(y0)) datbb <- datbb + aa[j]*datb[(i-j)]
        datb <- rbind(datb,datbb)
    }
    out <- tur_est(datb,p,mmin,mmax,m)  
    bootw[ib] <- out$w
    bootwt[ib] <- out$wtest
    bootws[ib,] <- t(out$ws)
    bootts[ib,] <- t(out$ts)
  }
  pw <- mean(bootw > w)
  pwt <- mean(bootwt > as.vector(wtest))
  pws <- as.matrix(colMeans(bootws > (matrix(1,boot,1)%*%t(ws))))
  pts <- as.matrix(colMeans(bootts > (matrix(1,boot,1)%*%t(ts))))
  for (k in 1:mn) bootws[,k] <- sort(bootws[,k])
  bootw <- sort(bootw)
  bootwt <- sort(bootwt)
  crlevel <- rbind(.9, .95, .99)
  crws <- t(bootws[round(boot*crlevel),])
  crw <- t(as.matrix(bootw[round(boot*crlevel)]))
  crwt <- t(as.matrix(bootwt[round(boot*crlevel)]))

## Bootstrap, Constrained Estimates ##
  aa <- matrix(0,p+1,1) 
  aa[1] <- 1
  aa[1:p] <- aa[1:p] + ar0
  aa[2:(p+1)] <- aa[2:(p+1)] - ar0
  bootw <- matrix(0,boot,1)
  bootws <- matrix(0,boot,mn)
  bootr1 <- matrix(0,boot,mn)
  bootr2 <- matrix(0,boot,mn)
  boott1 <- matrix(0,boot,mn)
  boott2 <- matrix(0,boot,mn)
  for (ib in 1:boot){
    eols_c <- as.matrix(eols[ceiling(runif(t)*n)])
    datb <- y0
    for (i in (nrow(y0)+1):(nrow(eols_c))){
        datbb <- eols_c[i]
        for (j in 1:nrow(y0)) datbb <- datbb + aa[j]*datb[(i-j)]
        datb <- rbind(datb,datbb)
    }
    out <- tur_est(datb,p,mmin,mmax,m)    
    bootw[ib] <- out$w
    bootws[ib,] <- t(out$ws)
    bootr1[ib,] <- t(out$r1)
    bootr2[ib,] <- t(out$r2)
    boott1[ib,] <- t(out$t1)
    boott2[ib,] <- t(out$t2)
  }
  pw_ur <- mean(bootw > w)
  pws_ur <- as.matrix(colMeans(bootws > (matrix(1,boot,1)%*%t(ws))))
  pr1 <- as.matrix(colMeans(bootr1 > (matrix(1,boot,1)%*%t(r1))))
  pr2 <- as.matrix(colMeans(bootr2 > (matrix(1,boot,1)%*%t(r2))))
  pt1 <- as.matrix(colMeans(boott1 > (matrix(1,boot,1)%*%t(t1))))
  pt2 <- as.matrix(colMeans(boott2 > (matrix(1,boot,1)%*%t(t2))))
  for (k in 1:mn){
    bootr1[,k] <- sort(bootr1[,k])
    bootr2[,k] <- sort(bootr2[,k])
    boott1[,k] <- sort(boott1[,k])
    boott2[,k] <- sort(boott2[,k])
  }
  crr1 <- t(bootr1[round(boot*crlevel),])
  crr2 <- t(bootr2[round(boot*crlevel),])
  crt1 <- t(boott1[round(boot*crlevel),])
  crt2 <- t(boott2[round(boot*crlevel),])

if (trend==0){
  if (trim==1){
    acrr1 <- cbind(10.84,12.75,16.97)
    acrr2 <- cbind(11.31,13.24,17.50)
    acrt  <- cbind(2.97,3.26,3.82)
    apr1 <- 1-pchisq(1.11284+1.13011*r1,8)
    apr2 <- 1-pchisq(-0.0107136+1.06417*r2,7)
    apt1 <- 1-pchisq(1.47612-0.0230281*t1+1.04816*(t1^2),6)
    apt2 <- 1-pchisq(1.47612-0.0230281*t2+1.04816*(t2^2),6)
  }
  if (trim==2){
    acrr1 <- cbind(11.09,13.00,17.23)
    acrr2 <- cbind(11.66,13.59,17.85)
    acrt <- cbind(3.01,3.31,3.85)
    apr1 <- 1-pchisq(0.959260+1.11866*r1,8)
    apr2 <- 1-pchisq(-0.261715+1.05436*r2,7)
    apt1 <- 1-pchisq(1.21165-0.561992*t1+1.07020*(t1^2),5)
    apt2 <- 1-pchisq(1.21165-0.561992*t2+1.07020*(t2^2),5)
  }
  if (trim==3){
    acrr1 <- cbind(11.35,13.29,17.51)
    acrr2 <- cbind(12.04,14.03,18.24)
    acrt <- cbind(3.05,3.34,3.89)
    apr1 <- 1-pchisq(0.783900+1.10665*r1,8)
    apr2 <- 1-pchisq(-0.572447+1.04393*r2,7)
    apt1 <- 1-pchisq(2.57297+1.63643*t1+1.03960*(t1^2),11)
    apt2 <- 1-pchisq(2.57297+1.63643*t2+1.03960*(t2^2),11)
  }
}

if (trend==1){
  if (trim==1){
    acrr1 <- cbind(14.06,16.24,20.75)
    acrr2 <- cbind(14.32,16.49,21.00)
    acrt <- cbind(3.40,3.69,4.26)
    apr1 <- 1-pchisq(0.455551+1.10432*r1,10)
    apr2 <- 1-pchisq(-0.284809+1.04342*r2,9)
    apt1 <- 1-pchisq(6.47943+3.82216*t1+0.974897*(t1^2),22)
    apt2 <- 1-pchisq(6.47943+3.82216*t2+0.974897*(t2^2),22)
  }
  if (trim==2){
    acrr1 <- cbind(14.30,16.44,20.98)
    acrr2 <- cbind(14.66,16.81,21.36)
    acrt <- cbind(3.45,3.73,4.29)
    apr1 <- 1-pchisq(0.281771+1.09842*r1,10)
    apr2 <- 1-pchisq(-0.0204306+1.09231*r2,10)
    apt1 <- 1-pchisq(5.93011+3.74162*t1+1.00551*(t1^2),22)
    apt2 <- 1-pchisq(5.93011+3.74162*t2+1.00551*(t2^2),22)
  }
  if (trim==3){
    acrr1 <- cbind(14.56,16.73,21.21)
    acrr2 <- cbind(15.07,17.22,21.73)
    acrt <- cbind(3.49,3.77,4.32)
    apr1 <- 1-pchisq(0.102428+1.0908*r1,10)
    apr2 <- 1-pchisq(-0.349898+1.08470*r2,10)
    apt1 <- 1-pchisq(4.96307+3.96004*t1+0.985536*(t1^2),22)
    apt2 <- 1-pchisq(4.96307+3.96004*t2+0.985536*(t2^2),22)
  }
}

## Print Output to Screen ##

nn <- rbind(sum(d1),sum(d2))
xname <- "Intercpt"
if (trend==1) xname <- rbind(xname,"Trend")
xname <- rbind(xname,"Y(t-1)","DY(t-1)")
for (k in 2:p){
    kname <- paste(c("DY(t-"),k,c(")"),sep="")
    xname <- rbind(xname,kname)
}
hname <- cbind("Variable","Estimate","St Error")
hname2 <- cbind(hname," ",hname[2],hname[3])
xname <- format(xname,digits=4)
hname <- format(hname,digits=4)
hname2 <- format(hname2,digits=4)
cat ("\n")
cat ("Number of Observations               ", n, "\n")
cat ("Lags of DY                           ", p, "\n")
cat ("Bootstrap Replications               ", boot, "\n")
if (thresh==1) cat ("Threshold is Long Difference", "\n")
if (thresh==2) cat ("Threshold is Lagged Difference", "\n")
if (thresh==3) cat ("Threshold is Lagged Level", "\n")
if (trim==1) cat ("Lambda Trimming Region: [.15,.85]", "\n")
if (trim==2) cat ("Lambda Trimming Region: [.10,.90]", "\n")
if (trim==3) cat ("Lambda Trimming Region: [.05,.95]", "\n")
cat ("\n")
cat ("\n")
cat ("Linear Model: ", "\n")
cat ("Residual Variance                    ", sigols, "\n")
cat ("ADF Statistic                        ", tadf, "\n")
cat ("\n")
cat (hname[1],"  ",hname[2],"   ",hname[3], "\n")
cat ("\n")
tbols <- format(bols,digits=4)
tseols <- format(seols,digits=4)
for (j in 1:nrow(bols)) cat(xname[j],"  ", tbols[j],"   ",tseols[j],"\n")
cat ("\n")
cat ("**********************************************************", "\n")
cat ("\n")
cat ("\n")
cat ("Threshold Model:", "\n")
if (m==0){ 
cat ("Estimated Delay Parameter         ", mhat, "\n")
cat ("(Minimizes sum-of-squared errors) ", "\n")
}else{
cat ("Delay Order                       ", m, "\n")
}
cat ("Residual Variance                 ", ee/(n-kx*2), "\n")
cat ("Gaussian Log-Likelihood           ", -(n/2)*(1+log(2*pi*ee/n))+adj, "\n")
cat ("\n")
cat ("Threshold Estimate                ", lam, "\n")
cat ("Observations in Regime 1 (%)      ", nn[1]," ",format((nn[1]/n),digits=4), "\n")
cat ("Observations in regime 2 (%)      ", nn[2]," ",format((nn[2]/n),digits=4), "\n")
cat ("\n")
cat ("\n")
cat ("            Regime 1 Estimates      Regime 2 Estimates ", "\n")
cat ("\n")
cat (hname2[1],"  ",hname2[2]," ",hname2[3],"   ",hname2[5]," ",hname2[6], "\n")
cat ("\n")
tbss <- format(bss,digits=4)
for (j in 1:nrow(bss)) cat(xname[j],"  ", tbss[j,1:2],"   ",tbss[j,4:5],"\n")
cat ("\n")
cat ("**********************************************************", "\n")
cat ("\n")
cat ("\n")
cat ("Wald Tests for Threshold Effect, Fixed Delay Parameter", "\n")
cat ("\n")
cat ("Delay  ","Wald   ","Boot.  ","Critical","Values ","Stat.  ","UnitRoot","\n")
cat ("Order  ","Stat   ","10%    ","5%      ","1%     ","P-Value","P-Value", "\n")
cat ("\n")
vs <- cbind(ms,ws,crws[,1],crws[,2],crws[,3],pws,pws_ur)
vs <- format(vs,digits=4)
for (j in 1:nrow(vs)) cat (vs[j,1]," ",vs[j,2]," ",vs[j,3]," ",vs[j,4]," ",vs[j,5]," ",vs[j,6]," ",vs[j,7],"\n")
cat ("\n")
cat ("\n")
if (mn>1){
  cat ("Wald Test for Threshold Effect, Estimated Delay Parameter", "\n")
  cat ("\n")
  cat ("Delay  ","Wald   ","Boot.  ","Critical","Values ","Stat.  ","UnitRoot","\n")
  cat ("Order  ","Stat   ","10%    ","5%      ","1%     ","P-Value","P-Value", "\n")
  cat ("\n")
  vs <- cbind(mhat,w,crw[,1],crw[,2],crw[,3],pw,pw_ur)
  vs <- format(vs,digits=4)
  cat (vs[1]," ",vs[2]," ",vs[3]," ",vs[4]," ",vs[5]," ",vs[6]," ",vs[7],"\n")
}  
cat ("\n")
cat ("**********************************************************", "\n")
cat ("\n")
cat ("\n")
cat ("Wald Tests for Threshold Effect in Individual Coefficients", "\n")
cat ("\n")
cat ("Variable   ","Wald      ","Bootstr   ","\n")
cat ("           ","Stat      ","P-Value   ","\n")
cat ("\n")
vs <- cbind(ts,pts)
vs <- format(vs, digits=4)
for (j in 1:nrow(vs)) cat(xname[j]," ",vs[j,1]," ",vs[j,2],"\n")
cat ("\n")
cat ("**********************************************************", "\n")
cat ("\n")
cat ("\n")
if (min(test)>0){
mt <- m
if (mt==0) mt <- mhat
cat ("Wald Test for Threshold Effect in Selected Coefficients", "\n")
cat ("\n")
cat ("Coefficients: ", t(test), "\n")
cat ("\n")
cat ("Delay    ","Wald     ","Boot.    ","Critical ","Values   ","Boot.    ","\n")
cat ("Order    ","Stat     ","10%      ","5%       ","1%       ","P-Value  ","\n")
cat ("\n")
vs <- cbind(mt,wtest,crwt[1],crwt[2],crwt[3],pwt)
vs <- format(vs, digits=4)
cat (vs[1],"  ",vs[2],"  ",vs[3],"  ",vs[4],"  ",vs[5],"  ",vs[6],"\n")
cat ("\n")
cat ("\n")
cat ("**********************************************************", "\n")
cat ("\n")
cat ("\n")
}
cat ("2-Sided Wald Test for Unit Root (R_2T)", "\n")
cat ("\n")
cat ("Asymptotic Critical Values", "\n")
cat ("10%    ","5%     ","1%   ","\n")
cat (acrr2[1]," ",acrr2[2]," ",acrr2[3],"\n")
cat ("\n")
cat ("Delay    ","Wald     ","Boot.    ","Critical ","Values   ","Boot.    ","Asym.    ","\n")
cat ("Order    ","Stat     ","10%      ","5%       ","1%       ","P-Value  ","P-Value  ","\n")
cat ("\n")
vs <- cbind(ms,r2,crr2[,1],crr2[,2],crr2[,3],pr2,apr2)
vs <- format(vs, digits=2)
for (j in 1:nrow(vs)) cat (vs[j,1],"  ",vs[j,2],"  ",vs[j,3],"  ",vs[j,4],"  ",vs[j,5],"  ",vs[j,6],"  ",vs[j,7],"\n")
cat ("\n")
cat ("\n")
cat ("1-Sided Wald Test for Unit Root (R_1T)", "\n")
cat ("\n")
cat ("Asymptotic Critical Values", "\n")
cat ("10%    ","5%     ","1%   ","\n")
cat (acrr1[1]," ",acrr1[2]," ",acrr1[3],"\n")
cat ("\n")
cat ("Delay    ","Wald     ","Boot.    ","Critical ","Values   ","Boot.    ","Asym.    ","\n")
cat ("Order    ","Stat     ","10%      ","5%       ","1%       ","P-Value  ","P-Value  ","\n")
cat ("\n")
vs <- cbind(ms,r1,crr1[,1],crr1[,2],crr1[,3],pr1,apr1)
vs <- format(vs, digits=2)
for (j in 1:nrow(vs)) cat (vs[j,1],"  ",vs[j,2],"  ",vs[j,3],"  ",vs[j,4],"  ",vs[j,5],"  ",vs[j,6],"  ",vs[j,7],"\n")
cat ("\n")
cat ("\n")
cat ("t Tests for Stationarity","\n")
cat ("\n")
cat ("Asymptotic Critical Values", "\n")
cat ("10%    ","5%     ","1%   ","\n")
cat (acrt[1]," ",acrt[2]," ",acrt[3],"\n")
cat ("\n")
cat ("\n")
cat ("t_1 Test", "\n")
cat ("\n")
cat ("Delay    ","t        ","Boot.    ","Critical ","Values   ","Boot.    ","Asym.    ","\n")
cat ("Order    ","Stat     ","10%      ","5%       ","1%       ","P-Value  ","P-Value  ","\n")
cat ("\n")
vs <- cbind(ms,t1,crt1[,1],crt1[,2],crt1[,3],pt1,apt1)
vs <- format(vs, digits=2)
for (j in 1:nrow(vs)) cat (vs[j,1],"  ",vs[j,2],"  ",vs[j,3],"  ",vs[j,4],"  ",vs[j,5],"  ",vs[j,6],"  ",vs[j,7],"\n")
cat ("\n")
cat ("\n")
cat ("t_2 Test", "\n")
cat ("\n")
cat ("Delay      ","t         ","Boot.     ","Critical  ","Values    ","Boot.     ","Asym.    ","\n")
cat ("Order      ","Stat      ","10%       ","5%        ","1%        ","P-Value   ","P-Value  ","\n")
cat ("\n")
vs <- cbind(ms,t2,crt2[,1],crt2[,2],crt2[,3],pt2,apt2)
vs <- format(vs, digits=4)
for (j in 1:nrow(vs)) cat (vs[j,1],"  ",vs[j,2],"  ",vs[j,3],"  ",vs[j,4],"  ",vs[j,5],"  ",vs[j,6],"  ",vs[j,7],"\n")
cat ("\n")
cat ("\n")
cat ("\n")
cat ("\n")
cat ("**********************************************************", "\n")
cat ("\n")
cat ("\n")
}





